@@ -11,10 +11,9 @@ implementation.
 import logging
 from os import getpid
 from weakref import ref
-from functools import partial
 
-from ctypes import Array, pointer, string_at, create_string_buffer, byref
-from ctypes import addressof, c_char_p, c_int, c_size_t, c_ulong
+from ctypes import Array, POINTER, cast, string_at, create_string_buffer, byref
+from ctypes import addressof, c_char_p, c_int, c_size_t, c_ulong, c_void_p, py_object
 from typing import Any, Callable, List, Optional, Sequence, Tuple
 from typing import cast as t_cast, TYPE_CHECKING
 
@@ -45,10 +44,9 @@ def version() -> int:
     return impl.PQlibVersion()
 
 
-def notice_receiver(
-    arg: Any, result_ptr: impl.PGresult_struct, wconn: "ref[PGconn]"
-) -> None:
-    pgconn = wconn()
+@impl.PQnoticeReceiver  # type: ignore
+def notice_receiver(arg: c_void_p, result_ptr: impl.PGresult_struct) -> None:
+    pgconn = cast(arg, POINTER(py_object)).contents.value()
     if not (pgconn and pgconn.notice_handler):
         return
 
@@ -57,8 +55,8 @@ def notice_receiver(
         pgconn.notice_handler(res)
     except Exception as exc:
         logger.exception("error in notice receiver: %s", exc)
-
-    res._pgresult_ptr = None  # avoid destroying the pgresult_ptr
+    finally:
+        res._pgresult_ptr = None  # avoid destroying the pgresult_ptr
 
 
 class PGconn:
@@ -70,7 +68,7 @@ class PGconn:
         "_pgconn_ptr",
         "notice_handler",
         "notify_handler",
-        "_notice_receiver",
+        "_self_ptr",
         "_procpid",
         "__weakref__",
     )
@@ -80,10 +78,9 @@ class PGconn:
         self.notice_handler: Optional[Callable[["abc.PGresult"], None]] = None
         self.notify_handler: Optional[Callable[[PGnotify], None]] = None
 
-        self._notice_receiver = impl.PQnoticeReceiver(  # type: ignore
-            partial(notice_receiver, wconn=ref(self))
-        )
-        impl.PQsetNoticeReceiver(pgconn_ptr, self._notice_receiver, None)
+        # Keep alive for the lifetime of PGconn
+        self._self_ptr = py_object(ref(self))
+        impl.PQsetNoticeReceiver(pgconn_ptr, notice_receiver, byref(self._self_ptr))
 
         self._procpid = getpid()
 
@@ -529,6 +526,7 @@ class PGconn:
             )
 
     def flush(self) -> int:
+        # PQflush segfaults if it receives a NULL connection
         if not self._pgconn_ptr:
             raise e.OperationalError("flushing failed: the connection is closed")
         rv: int = impl.PQflush(self._pgconn_ptr)
@@ -560,8 +558,7 @@ class PGconn:
         else:
             return None
 
-    def put_copy_data(self, buffer: bytes) -> int:
-        # TODO: should be done without copy
+    def put_copy_data(self, buffer: "abc.Buffer") -> int:
         if not isinstance(buffer, bytes):
             buffer = bytes(buffer)
         rv = impl.PQputCopyData(self._pgconn_ptr, buffer, len(buffer))
@@ -842,7 +839,11 @@ class PGcancel:
         See :pq:`PQcancel()` for details.
         """
         buf = create_string_buffer(256)
-        res = impl.PQcancel(self.pgcancel_ptr, pointer(buf), len(buf))  # type: ignore
+        res = impl.PQcancel(
+            self.pgcancel_ptr,
+            byref(buf),  # type: ignore[arg-type]
+            len(buf),
+        )
         if not res:
             raise e.OperationalError(
                 f"cancel failed: {buf.value.decode('utf8', 'ignore')}"
@@ -870,7 +871,7 @@ class Conninfo:
             raise TypeError(f"bytes expected, got {type(conninfo)} instead")
 
         errmsg = c_char_p()
-        rv = impl.PQconninfoParse(conninfo, pointer(errmsg))
+        rv = impl.PQconninfoParse(conninfo, byref(errmsg))  # type: ignore[arg-type]
         if not rv:
             if not errmsg:
                 raise MemoryError("couldn't allocate on conninfo parse")
@@ -954,10 +955,10 @@ class Escaping:
             out = create_string_buffer(len(data) * 2 + 1)
             impl.PQescapeStringConn(
                 self.conn._pgconn_ptr,
-                pointer(out),  # type: ignore
+                byref(out),  # type: ignore[arg-type]
                 data,
                 len(data),
-                pointer(error),
+                byref(error),  # type: ignore[arg-type]
             )
 
             if error:
@@ -968,7 +969,7 @@ class Escaping:
         else:
             out = create_string_buffer(len(data) * 2 + 1)
             impl.PQescapeString(
-                pointer(out),  # type: ignore
+                byref(out),  # type: ignore[arg-type]
                 data,
                 len(data),
             )
@@ -988,10 +989,14 @@ class Escaping:
                 self.conn._pgconn_ptr,
                 data,
                 len(data),
-                pointer(t_cast(c_ulong, len_out)),
+                byref(t_cast(c_ulong, len_out)),  # type: ignore[arg-type]
             )
         else:
-            out = impl.PQescapeBytea(data, len(data), pointer(t_cast(c_ulong, len_out)))
+            out = impl.PQescapeBytea(
+                data,
+                len(data),
+                byref(t_cast(c_ulong, len_out)),  # type: ignore[arg-type]
+            )
         if not out:
             raise MemoryError(
                 f"couldn't allocate for escape_bytea of {len(data)} bytes"
@@ -1008,7 +1013,10 @@ class Escaping:
             self.conn._ensure_pgconn()
 
         len_out = c_size_t()
-        out = impl.PQunescapeBytea(data, pointer(t_cast(c_ulong, len_out)))
+        out = impl.PQunescapeBytea(
+            data,
+            byref(t_cast(c_ulong, len_out)),  # type: ignore[arg-type]
+        )
         if not out:
             raise MemoryError(
                 f"couldn't allocate for unescape_bytea of {len(data)} bytes"
